PROGRAM Example4;
{
        This is an example of the SuperFX unit

        This is the same demo as example1, but this one uses the XMS version.
        It's a bit slower then the other one, but it has a lot more memory
        to play with.

        Copyright 1996,1997 by Just4Fun Productions

        A simple platform walking happening...
        No checking on "falling" of the screen, so once you fall, you have
        to restart.

        Features shown: - all directions scrolling
                        - 4 multiple layers of parallax
                        - infront "background" map tiles
                        - how to check on background collision
                          (just look at player.detection)
                        - screen zoom/sprite zoom
                        - fadein, fadeout
                        - the displacement procedure (see intro part)
                        - grey mode, or color mode (change status to SAD or
                          HAPPY)
                        - joystick/keyboard control
                        - joystick calibrating
                        - how to load the config file created with SETUP
                        - showing a HIGH screen image (360x480x256)

        Controls        : Use setup to select the movement controls,
                          CTRL to do a screen-wave
                          ALT  to flip the screen
                          M    to switch between SAD and HAPPY mode

}


{$DEFINE USEDOSMEM }      { Use DOS to allocate memory... }

{$IFDEF USEDOSMEM}
   {$M 16384,0,5000}     { Minimum of HEAP memory needed }
{$ELSE}
   {$M 65520,0,655360}   { Maximum of HEAP memory needed }
{$ENDIF}

USES CRT,DOS,SFX_XMS,SFX_VARS;

CONST GameVersion = $003;               { game version                }
      CFGfile     = 'default.cfg';      { use the default config file }
      ControlType :byte = 1;            { 1-keyboard 2-joystick,read from CFG}
{}
{

        Misc variables

}
TYPE StatusType = (Sad,Happy);
VAR  Status              : StatusType;
     StatusDelayPressing : byte;
     JoyPic              : array[1..2] of SpriteTyp;
{}
{

    Message Zooming variables

}
VAR   MessageZoomSize : byte;           { counter for size of zooming        }
      MessageNumber   : byte;           { number of the message sprite       }
      ZoomIT          : boolean;        { zooming into a sprite?             }
      MessageImages   : array[0..5] of SpriteTyp; { all the zoom messages    }
      flipY           : byte;
      flipA           : byte;
{}
{

    Player Ojbect - for PlatForm Games

}
TYPE TPlayerPlatForm   =  object
      DownPressed        : boolean;     { TRUE if pressed down to jump below }
      Xposition          : word;        { X position on the map in pixels }
      Yposition          : word;        { Y position on the map in pixels }
      Flipped            : boolean;     { Using mirrored images? }
      ground             : boolean;     { player walking on ground? }
      GroundFall         : boolean;     { if hit ground }
      Yspeed,Xspeed      : real;        { X and Y speed }

      score              : longint;     { take a guess?! }
      ScoreTXT           : string[9];   { the score is 9 digits }

      CurrentIMG         : byte; { Current animation frame }
        PROCEDURE Init;          { initialise the position, and variables }
        PROCEDURE Detection;     { detection of map-blocks (ground,wall,etc) }
        PROCEDURE Movement(tleft,tright,tup,tdown,tfire:boolean);
        PROCEDURE AddScore(points:word); { add points to the score }
        PROCEDURE ShowScore(x,y:word);   { show the score at X,Y position }
     end;

VAR  PlayerImages : array[0..10] of SpriteTyp;{ the sprite-data of the player}
     Player       : TplayerPlatform;          { a complete PLAYER! }
{}
{
        Initialise the Player variables, and the map-position

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE TPlayerPlatform.Init;
BEGIN
    Xposition:=PlayerStartX shl TileShiftFactor;
    Yposition:=PlayerStartY shl TileShiftFactor;
    Flipped:=false;
    Ground:=false;
    GroundFall:=false;
    score:=000000000;
    ScoreTXT:='000000000';
    Xspeed:=0;
    Yspeed:=0;
    CurrentImg:=1;
    ASM
       mov ax,player.Xposition
       sub ax,160
       mov mapx,ax
       mov bx,player.Yposition
       sub bx,100
       mov mapY,bx
       Call Map_Calculate
    end;
    {

        See for ground below

    }
    with PLAYER do begin
         CASE mapPos((xposition shr TileShiftFactor)+1,yposition shr TileShiftFactor,mapBitFlag) of
         1..2 : BEGIN
                 Yposition:=Yposition shr TileShiftFactor;
                 inc(Yposition);
                 Yposition:=Yposition shl TileShiftFactor;
                 dec(Yposition,PlayerImages[0].height);
                END;
         end;
    end;
END;
{}
{
        Add points to the score (score value, and scoreTEXT will be updated)

        Expects: Amount of points to increase the score with
        Returns: Nothing
}
PROCEDURE TPlayerPlatform.AddScore(points:word);
BEGIN
 inc(score,points);                     { increase score counter             }
 str(score,scoreTXT);                   { set it into a string only once     }
 fillchar(scoreTXT[1],9-length(scoreTXT),'0');   { add some zeros infront    }
 scoreTXT[0]:=char(9);                  { make sure it's only 9 characters   }
END;
{}
{
        Show the score of the player

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE TPlayerPlatform.ShowScore(x,y:word);
BEGIN
 TEXT(x,y,scoreTXT);    { simply put the score on screen }
END;
{}
{
        Do detection on the player hitting background objects

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE TplayerPlatform.Detection;
VAR ty2,ty3,tx2,tx3   : word;
    x,y               : word;
    ctop              : boolean;
    cbot              : boolean;
    cright            : boolean;
    cleft             : boolean;
    cslope            : boolean;
    wi,hi             : byte;
    xt,yt,
    mapt              : integer;
    bitflag,f_Frame   : byte;
BEGIN
 ASM
  mov ax,player.yposition
  shr ax,TileshiftFactor
  cmp ax,0
  jng @nodecreaseY
  dec ax
 @nodecreaseY:
  mov ty2,ax
  add ax,2
  mov ty3,ax
  mov ax,player.xposition
  shr ax,TileShiftFactor
  cmp ax,0
  jng @nodecreaseX
  dec ax
 @nodecreaseX:
  mov tx2,ax
  add ax,2
  mov tx3,ax

  mov cslope,false
  mov ctop,false
  mov cbot,false
  mov cleft,false
  mov cright,false
  mov cleft,false
 END;

 { Get width and height of player image }
 wi:=PlayerImages[currentIMG].width;
 hi:=PlayerImages[currentIMG].height;

 FOR x:=tx2 to tx3 do begin
   xt:=x shl TileShiftFactor; { calculate X shl 5 only onces per vertical Row! }
   FOR y:=ty2 to ty3 do begin
   {==========================================================================
        Now get the blocks above,below,left and on the right side of the
        player
   ==========================================================================}
       BitFlag:=mapPos(x,y,mapBitflag);
       f_frame:=mapPos(x,y,mapF_frame);
        yt:=y shl TileShiftFactor;
        {=====================================================================
            See if this block is not just empty-air
        =====================================================================}
        { Some testing now... }
        IF BITFLAG<>0 THEN BEGIN
           {==================================================================
              See player position (block behind the player)
           ==================================================================}
           if (collision( xt,yt,
                          tiles[f_Frame].width, tiles[f_Frame].height,
                          xposition,yposition+1,wi,hi))
                          then begin
                             CASE BITFLAG of
                             { ********************
                                 Secret IDentifier
                             **********************}
                                  9 : begin
                                         if not zoomIT then begin
                                            ZoomIT:=true;
                                            MessageZoomSize:=2;
                                            MessageNumber:=1;
                                         end;
                                      end;

                             end; { case blocktype }
           end;

           {==================================================================
              See Top of player
           ==================================================================}
           IF NOT CTOP then
           if (collision( xt+5,
                          yt+tiles[f_Frame].height-5,
                          tiles[f_Frame].width-10,5,
                          xposition,yposition,wi,10))
                          then begin
                             CASE BITFLAG of
                             { *****************
                                 Massive Block
                             ******************}
                                  1 : begin
                                         yposition:=1+yt+tiles[f_Frame].height;
                                         yspeed:=0.2;
                                         ground:=false;
                                         ctop:=true;
                                      end;
                             end; { case blocktype }
           end;
           {==================================================================
              See Bottom of player
           ==================================================================}
           IF NOT cbot then
           if (yspeed>0.0) and  (collision( xt+5,yt-1,tiles[f_Frame].width-10,7,
                           xposition,yposition+hi-5,wi,5))
                           then begin
                             CASE BITFLAG of
                             { **************
                                 Walk able
                             ****************}
                                  1  : begin
                                          yposition:=yt-hi;
                                          Yspeed:=0;
                                          ground:=true;
                                          cbot:=true;
                                         end;
                                  2  : begin
                                         if not DownPressed then begin
                                            yposition:=yt-hi;
                                            Yspeed:=0;
                                            ground:=true;
                                            cbot:=true;
                                         end;
                                       end;
                             end; { case blocktype }
           end;
           {==================================================================
              See Right Side of the player
           ==================================================================}
           if not cright then
           if (xspeed>0) and (collision(xt,yt+5,5,tiles[f_Frame].height-10,
                         xposition+wi,yposition,1,hi)) then begin
              CASE BITFLAG of
              {*******************
                   Massive Block
              ********************}
                1 : begin
                     xposition:=xt-wi;
                     xspeed:=0;
                     cright:=true;
                    end;
              end; { case blocktype }
           end;
           {==================================================================
              See Left Side of the player
           ==================================================================}
           if not cleft then
           if (xspeed<0) and
              (collision(xt,yt+5,tiles[f_Frame].width,
                         tiles[f_Frame].height-10,
                         xposition,yposition,1,hi)) then begin
              CASE BITFLAG of
              {*******************
                  Massive Block
              ********************}
                1 : begin
                     xposition:=xt+tiles[f_Frame].width;
                     xspeed:=0;
                     cleft:=true;
                    end;
              end; { case blocktype }
           end; { end left side }
    { --------------------------------------------------------------------- }
    END; { end collision_detection }
  END; { Y counter }
 END; { X counter }
END;
{}
{
        Do the player movements

        Expects: Nothing
        Returns: Nothing
}

PROCEDURE TPlayerPlatform.Movement;
BEGIN
   Ground:=false;
   Detection;
   if not GROUND then begin
      if yspeed<6 then yspeed:=yspeed+0.3;
   end else DownPressed:=false;
   WITH PLAYER DO BEGIN
        IF Tleft then begin
                 if not Flipped then Flipped:=true;
                 if xspeed>-4 then xspeed:=xspeed-0.2;
                 if ground then begin
                    dec(currentIMG);
                    if currentIMG=255 then CurrentIMG:=playerImages[0].a_frame-1;
                 end;
        end else IF Tright then begin
                 if FLIPPED then Flipped:=false;
                 if xspeed<4 then xspeed:=xspeed+0.2;
                 if ground then begin
                    inc(currentIMG);
                    if currentIMG>playerImages[0].a_frame-1 then CurrentIMG:=0;
                 end;
        end else begin
                 if xspeed>0.0 then xspeed:=xspeed-0.2
                 else
                   if xspeed<0.0 then xspeed:=xspeed+0.2
                 else xspeed:=0;
               CurrentIMG:=playerImages[0].a_frame+2
        end;

        IF (round(xspeed)=0) and (round(Yspeed)=0) then begin
           CurrentIMG:=playerImages[0].a_frame+2;
           if Flipped then begin
              if mapPos((xposition shr TileShiftFactor)+1,(yposition shr TileShiftFactor),mapBitFlag)=0 then inc(CurrentImg);
           end else begin
              if (mapPos((xposition shr TileShiftFactor)+1,(yposition shr TileShiftFactor)+1,mapBitFlag)=0) and
                 (xposition and 31>15) then inc(CurrentImg);
           end;
        end else
           if round(yspeed)<>0 then currentImg:=playerImages[0].a_frame;

        IF TUp and GROUND then begin
           yspeed:=-7.6;
           GroundFall:=true;
        END;

        IF TDown then DownPressed:=true else
           if yspeed>4 then DownPressed:=false;

        Xposition:=Xposition+round(Xspeed);
        if (xposition-mapx)>294 then begin
           if xspeed<0 then xposition:=0 else xposition:=294+mapx;
           xspeed:=0;
        end;
        Yposition:=Yposition+round(Yspeed);
        asm
         mov ax,player.Xposition
         sub ax,155
         mov mapx,ax
         mov bx,player.Yposition
         sub bx,95
         mov mapY,bx
         Call Map_Calculate
        end;
        AddIMAGE( Player.Xposition-MAPX,Player.Yposition-MAPY,
                  PlayerImages[CurrentImg],Flipped,false);
   END;
END;
{}
{
      Load the configuration ( using the SETUP program wich came with the
      SuperFX Engine)

      Expects: Nothing
      Returns: Nothing
}
PROCEDURE LoadCFG;
VAR f  :  file;
    SoundCardV : byte;          { we just use these for "fun" }
    PortV      : byte;
    IRQV       : byte;
    DMAV       : byte;
    SAMPLEV    : byte;
BEGIN
  assign(f,CFGFile);
  {$I-}
    reset(f,1);
  {$I+}
   if ioresult<>0 then exit;
      if not eof(f) then blockread(f,controlType,1);
      if not eof(f) then blockread(f,SoundCardV,1);
      if not eof(f) then blockread(f,PortV,1);
      if not eof(f) then blockread(f,IRQV,1);
      if not eof(f) then blockread(f,DMAV,1);
      if not eof(f) then blockread(f,SAMPLEV,1);

      {
        These values are directly read into the KEY array, so you can
        program using the "keys" LEFT,RIGHT,UP,DOWN, and the configuration
        will simple change these constants to the correct key-value
      }
      if not eof(f) then blockread(f,LEFT,1);
      if not eof(f) then blockread(f,RIGHT,1);
      if not eof(f) then blockread(f,DOWN,1);
      if not eof(f) then blockread(f,UP,1);

      if not eof(f) then blockread(f,JoyLeft,1);
      if not eof(f) then blockread(f,JoyRight,1);
      if not eof(f) then blockread(f,JoyDown,1);
      if not eof(f) then blockread(f,JoyUp,1);
  close(f);
END;
{}
{
        Show the level start, in a very flashy way

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE PopLevel;
VAR Ypos,Yspeed  : integer;
    YInitSpd     : integer;
    dataptr      : pointer;
    temp_seg     : word;
    donefirstbounce : Boolean;
    xtarget         : integer;
BEGIN
 temp_seg:=vpage_Seg;   { remember current virtual page segment }
 {

   Initialise the screen

 }
 ClearScreen;
 ClearVisible;
 {

    Calculate the map

 }
 ASM
    mov ax,player.Xposition
    sub ax,155
    mov mapx,ax
    mov bx,player.Yposition
    sub bx,95
    mov mapY,bx
    Call Map_Calculate
 END;

 {

   Let the player run into the screen

 }
 xtarget:=mapx-player.xposition+50;
 player.xspeed:=4;
 REPEAT
   if xtarget<player.xposition-20 then begin
      inc(player.currentIMG);
      if player.currentIMG>playerImages[0].a_frame-1 then player.CurrentIMG:=0;
   end else player.currentIMG:=playerImages[0].a_frame+2;

   xtarget:=xtarget+round(player.xspeed);
   if xtarget>player.xposition-20 then
      if player.xspeed>0.4 then player.xspeed:=player.xspeed-0.4 else player.xspeed:=0;

    XMS2RAM( XMShandle,PlayerImages[player.CurrentIMG].spriteofs,XMSsprite,
             PlayerImages[player.CurrentIMG].size);
   {

     Draw player image

   }
   ASM
    mov ax,xtarget
    mov bx,player.Yposition
    sub bx,MapY
    sub ax,MapX
    les DI,XMSsprite
    Call Sprite
   END;

   Retrace;
   ShowBitmap;
   ClearScreen;
 UNTIL player.xspeed=0.0;
 {

     Draw the map

 }
 vpage_seg:=temp_seg;
 UpdateMap;

 XMS2RAM(XMShandle,PlayerImages[PlayerImages[0].a_Frame+2].spriteofs,
         XMSsprite,PlayerImages[PlayerImages[0].a_Frame+2].size);
 {

   Now bounce the screen in

 }
 Ypos:=199;
 Yspeed:=-25;
 YInitSpd:=-17;
 DoneFirstBounce:=false;

 REPEAT
    ASM
      call retrace
      xor ax,ax
      mov bx,Ypos
     call Displace
    END;
    {

       Now to show the player in the center (screen bouncing behind him)

    }
   vpage_seg:=sega000;    { set it to visible screen }
   IF Player.FLipped then { flipped image? }
      ASM
        mov ax,player.Xposition
        mov bx,player.Yposition
        sub bx,MapY
        sub ax,MapX
        les DI,XMSsprite
        Call XFlipSprite;
      END
   else
      ASM               { non flipped image }
        mov ax,player.Xposition
        mov bx,player.Yposition
        sub bx,MapY
        sub ax,MapX
        les DI,XMSsprite
        Call Sprite;
      END;
   vpage_seg:=temp_seg;  { get segment of virtual page back }

   inc(Ypos,Yspeed);
   inc(Yspeed);
   if ypos<0 then DoneFirstBounce:=true;
   if (ypos>=0) and (DoneFirstBounce) then begin
      ypos:=0;
      Yspeed:=YInitSpd;
      inc(YInitSpd,4);
   end;
 UNTIL (YInitSpd>=0) or (key[esckey]);
END;
{}
{
        Zoom a message-sprite

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE ZoomMessage;
BEGIN
  XMS2RAM( XMShandle,MessageImages[MessageNumber].spriteofs,XMSsprite,
           MessageImages[MessageNumber].size);
  ScaleShape( XMSsprite^,
              160-(MessageZoomsize shr 1),100-(MessageZoomsize shr 1),
              160+(MessageZoomsize shr 1),100+(MessageZoomsize shr 1));
  inc(MessageZoomsize,4);
  if MessageZoomsize>150 then begin
     MessageZoomsize:=2;
     ZoomIT:=false;
  end;
END;
{}
VAR i,j  : word;      { some variables to use with the zoom                  }
{}
{       LET'S START THIS EXAMPLE!!!                                          }
BEGIN
 write('Locating library :');
   { First see if the datafile is present }
   IF not UseFile('SUPERFX.DFP') then begin
      halt_Error:='ERROR - File not found:SUPERFX.DFP';   { oops, not found! }
      halt;                                               { just halt        }
   end;
 writeln('ok');
 write('Loading config :');
   LoadCFG; write('.');                     { load the configuration file    }
 writeln('ok');

 writeln;
 writeln('press a key');
 repeat until anypressed;
 blackpal;                                { make sure the palette is black   }
 LoadPCXhigh(true,'sfxlogo.pcx');          { load in a HIGH screen            }

 GetXas(true,'tiles.xas',tiles);          { load the tiles                   }
 GetXas(true,'message.xas',messageImages);{ read zoom messages               }
 LoadLAyerImage(true,'parallax.xas',0);   { read parallax layers             }
 LoadCharset(true,'charset.chr');         { read in a charset                }
 GetXas(true,'mayhem.xas',PlayerImages);  { read main player images          }
 if not LoadMap(true,'game.map') then begin { load a MAP file                }
    setTXTmode;                           { couldn't find a map              }
    halt;                                 { HALT the program!                }
 end;
{

   Now, we will initialise a few things first

}
 FadeOut(0,255);                          { fade out the screen              }
 InitVGAmode;                             { Initialise VGA mode              }
 LoadPal(true,'palette.col');         { load the palette                 }

 Player.Init;          { initialise the player                          }

 Status:=HAPPY;        { change this to SAD for a black/white version   }

 SETPAL;               { set the correct pal                            }
 if status=SAD then GreyPal(0,247); { if SAD mode, make it GREY         }

 {

    The next part will "swirl-calibrate" the joystick, if used...

 }
 if ControlTYPE=2 then begin { Calibrate the joystick                   }
    GETXAS(true,'joystick.xas',joypic);  { read joystick images         }
    asm
     call setpal                { set the palette we loaded             }
     mov ax,127                 { X position of image                   }
     mov bx,67                  { Y position of image                   }
     mov si,offset joypic       { first image                           }
     les di,[ds:si]             { get pointer to data                   }
     call SPRITENOMASK          { draw the sprite, no masking           }
     call ShowBitmap            { show the bitmap (image)               }
    end;
    REPEAT SAMPLE; UNTIL JoystickA or JoystickB or JoystickC or JoystickD;
    asm
     mov ax,127                 { X position of image                   }
     mov bx,67                  { Y position of image                   }
     mov si,offset joypic       { get image offset                      }
     add si,10                  { and get data of second image          }
     les di,[ds:si]             { get pointer to data                   }
     call SPRITENOMASK          { draw the sprite, no masking           }
     call ShowBitmap            { show the bitmap (image)               }
    end;
    SwirlCalibrate;             { Do the "swirl" thing                  }
 end;


 {
    Now, we start the level
 }
{ PopLevel;             { popin the level                                }
 {
    Show the GO! message
 }
 MessageZoomsize:=2;   { start with size 2                              }
 ZoomIT:=true;         { zoomIT = true                                  }
 MessageNumber:=0;     { show message GO                                }

 RestoreTimerInterrupt;

 REPEAT

  CASE ControlTYPE of
   1 : Player.Movement(key[left],key[right],key[up],key[down],key[ctrl]);
   2 : BEGIN
          GetJoy;
          Player.Movement(Joystickleft,joystickright,joystickUp,joystickdown,JoystickB);
   end;
  END;

  UpdateMap;

  TestCpuLine;
  if ZoomIT then ZoomMessage;

   if key[alt] then FlipSCreenY else
      if FXWave then WaveFX else ShowBitmap;

   IF StatusDelayPressing>0 then dec(StatusDelayPressing) else
      IF key[ 50{M} ] then begin
         case STATUS of
             HAPPY : begin
                       STATUS:=SAD;
                       GreyPal(0,247);
                       StatusDelayPressing:=30;
             end;
             SAD   : begin
                       STATUS:=HAPPY;
                       SetPal;
                       StatusDelayPressing:=30;
             end;
         end;
   END;

   {

      Now show the status bar stuff, so it won't WAVE with the reset
      of the screen, giving an extra dimension effect

   }
   Player.ShowScore(4,185);
   TEXT(4,175,'SCORE');
   TEXT(220,175,'TIME');
   TEXT(280,175,'1UP');
   CASE Status of
    SAD   : TEXT(140,175,'MAGIC');
    HAPPY : TEXT(140,175,'STARS');
   END;

   IF key[CTRL] then InitWave(5);

 UNTIL (key[esckey]);
 SetTXTMode;
END.

