PROGRAM CHMAKE;   { Convert PCX files to a charset file }

USES Crt,
     Dos,
     param_ch;

CONST version = 'v1.0';
      blocksize = 40000;
      id        : string ='CHR';

TYPE SpriteTyp  = record
        Adr     : Pointer;
        dtx,dty : word;
     end;

VAR sprite      : SpriteTyp;
    mask        : string;
    filename    : string;
    f1,f2,temp  : file;
    files       : array[0..500] of string[12];

    gfx_Error   : string;

    take1,                                 {used in the copy parts}
    take2      : longint;                  {...see above...}
    bcopy1     : array[1..BLOCKSIZE] of byte;  {and again...}

{[ See if a file is present ]}
FUNCTION seefile(name:string):boolean;
{Look for a certain file}
VAR f : file;
BEGIN
 seefile:=false;
 assign(f,name);
 {$I-}
  reset(f);
  close(f);
  if ioresult=0 then seefile:=true;
 {$I+}
END;
{}
{
        Decode a PCX file

        Returns    : a VAR spritetyp

        Parameters : Lib       = True if library file use, else FALSE
                     Filename  = Spritename, file or key in lib file
                 VAR Spt       = Returned sprite-image

}
PROCEDURE decode(lib:boolean;filename:string;var spt:SpriteTyp);
TYPE PCX_Header = record
       manu     : byte; { creator }
       version  : byte; { Version }
       encoding : byte;
       b_p_pix  : byte;
       xmin     : word;
       ymin     : word;
       xmax     : word;
       ymax     : word;
       hres     : word;
       vres     : word;
       colormap : array[1..48] of byte;
       reserve  : byte;
       nplanes  : byte;
       bpline   : word;
       palinfo  : word;
       filler   : array[1..58] of byte;
    end;

var  x,y         : word;
     msize       : word;         { to allocate memory }
     Xsize,Ysize : word;         { Size of image }
     Xscrn,Yscrn : word;         { Screen positions }
     plotdata    : byte;         { Byte-Color-Pixel to plot }
     readbyte    : byte;         { Byte to read from file }
     count       : integer;         { Count of continuous bytes }
     count2      : integer;         { Count from 1 to 'COUNT'   }
     Header      : PCX_Header;
     code        : integer;
     lib_file    : file;
BEGIN
  assign(lib_file,filename); reset(lib_file,1);

  blockread(lib_file,header,sizeof(header));      { Read header first}

 { Check for correct PCX format }
 if header.encoding<>1 then gfx_error:='Weird encoding used.';
 if header.b_p_pix<>8 then gfx_error:='To many bits per pixel.';
 if header.nplanes<>1 then gfx_error:='To many color-planes used.';
 if gfx_error<>'' then begin
    close(lib_file);         { Close the file }
    exit;                    { And exit this procedure }
 end;

  { Init some values }
  Xsize:=header.xmax-header.xmin+1;             { Get actual X-Size of image }
  Ysize:=header.ymax-header.ymin+1;             { Get actual Y-Size of image }
  Xscrn:=0;                                     { Screen positions }
  Yscrn:=0;                                     { Screen positions }
  spt.dtx:=xsize;
  spt.dty:=ysize;

  { get memory for sprite }
  msize:=(xsize*ysize)+2;
  if (msize mod 16)<>0 then msize:=msize+16;
  msize:=16*(msize div 16);

  getmem(spt.adr,msize);
  mem[seg(spt.adr^):ofs(spt.adr^)]:=byte(xsize);
  mem[seg(spt.adr^):ofs(spt.adr^)+1]:=byte(ysize);

  if spt.adr=nil then begin
     gfx_error:='Not enough memory for sprite.';
     exit;
  end;

   REPEAT
       blockread(lib_file,readbyte,1);            { Read in next byte }
       if (readbyte and 192)=192 then begin     { Are Top 2 bits set? }
           count:=readbyte and 63;              { Yes, so it's a packed }
           blockread(lib_file,plotdata,1);        { sequence. Read in color }
       end else begin
           count:=1;                            { Not packed, so 1 byte to set}
           plotdata:=readbyte;                  { Set the color to plot }
       end;
       if count>0 then begin
           fillchar( mem[seg(spt.adr^):ofs(spt.adr^)+2
                     +(Yscrn*xsize)+Xscrn],count,plotdata);
       end;
       inc(xscrn,count);
       if xscrn>=xsize then begin             { Hit the image-width? }
          xscrn:=0;                           { Yes, so restart at 0 }
          inc(yscrn);                         { And go 1 line down on screen}
       end;
  UNTIL (yscrn=ysize);                           { Repeat until complete image}
  close(lib_file);                                 { Close image file }
END;

{}
function WildComp(NameStr,SearchStr: String): Boolean; assembler;
{
 Compare SearchStr with NameStr, and allow wildcards in SearchStr.
 The following wildcards are allowed:
 *ABC*        matches everything which contains ABC
 [A-C]*       matches everything that starts with either A,B or C
 [ADEF-JW-Z]  matches A,D,E,F,G,H,I,J,W,V,X,Y or Z
 ABC?         matches ABC, ABC1, ABC2, ABCA, ABCB etc.
 ABC[?]       matches ABC1, ABC2, ABCA, ABCB etc. (but not ABC)
 ABC*         matches everything starting with ABC
 (for using with DOS filenames like DOS (and 4DOS), you must split the
  filename in the extention and the filename, and compare them seperately)
}
var
  LastW: word;
asm
  cld
  push ds
  lds si,SearchStr
  les di,NameStr
  xor ah,ah
  lodsb
  mov cx,ax
  mov al,es:[di]
  inc di
  mov bx,ax
  or cx,cx
  jnz @ChkChr
  or bx,bx
  jz @ChrAOk
  jmp @ChrNOk
  xor dh,dh
 @ChkChr:
  lodsb
  cmp al,'*'
  jne @ChkQues
  dec cx
  jz @ChrAOk
  mov dh,1
  mov LastW,cx
  jmp @ChkChr
 @ChkQues:
  cmp al,'?'
  jnz @NormChr
  inc di
  or bx,bx
  je @ChrOk
  dec bx
  jmp @ChrOk
 @NormChr:
  or bx,bx
  je @ChrNOk
 {From here to @No4DosChr is used for [0-9]/[?]/[!0-9] 4DOS wildcards...}
  cmp al,'['
  jne @No4DosChr
  cmp word ptr [si],']?'
  je @SkipRange
  mov ah,byte ptr es:[di]
  xor dl,dl
  cmp byte ptr [si],'!'
  jnz @ChkRange
  inc si
  dec cx
  jz @ChrNOk
  inc dx
 @ChkRange:
  lodsb
  dec cx
  jz @ChrNOk
  cmp al,']'
  je @NChrNOk
  cmp ah,al
  je @NChrOk
  cmp byte ptr [si],'-'
  jne @ChkRange
  inc si
  dec cx
  jz @ChrNOk
  cmp ah,al
  jae @ChkR2
  inc si              {Throw a-Z < away}
  dec cx
  jz @ChrNOk
  jmp @ChkRange
 @ChkR2:
  lodsb
  dec cx
  jz @ChrNOk
  cmp ah,al
  ja @ChkRange        {= jbe @NChrOk; jmp @ChkRange}
 @NChrOk:
  or dl,dl
  jnz @ChrNOk
  inc dx
 @NChrNOk:
  or dl,dl
  jz @ChrNOk
 @NNChrOk:
  cmp al,']'
  je @NNNChrOk
 @SkipRange:
  lodsb
  cmp al,']'
  loopne @SkipRange
  jne @ChrNOk
 @NNNChrOk:
  dec bx
  inc di
  jmp @ChrOk
 @No4DosChr:
  cmp es:[di],al
  jne @ChrNOk
  inc di
  dec bx
 @ChrOk:
  xor dh,dh
  dec cx
  jnz @ChkChr        { Can't use loop, distance >128 bytes }
  or bx,bx
  jnz @ChrNOk
 @ChrAOk:
  mov al,1
  jmp @EndR
 @ChrNOk:
  or dh,dh
  jz @IChrNOk
  jcxz @IChrNOk
  or bx,bx
  jz @IChrNOk
  inc di
  dec bx
  jz @IChrNOk
  mov ax,[LastW]
  sub ax,cx
  add cx,ax
  sub si,ax
  dec si
  jmp @ChkChr
 @IChrNOk:
  mov al,0
 @EndR:
  pop ds
end;
{}
    PROCEDURE sort_it(first,last:byte);
    var temp : string[12];
        zin1,zin2 : string;
        x,i,j  : byte;
    begin
     for i:=first to last do begin
         for j:=first to last-1 do begin
             x:=pos('.',files[j+1]);
             if x<>0 then zin1:=copy(files[j+1],1,x-1) else zin1:=files[j+1];
             x:=pos('.',files[j]);
             if x<>0 then zin2:=copy(files[j],1,x-1) else zin2:=files[j];
           if zin1<zin2 then begin
              temp:=files[j];
              files[j]:=files[j+1];
              files[j+1]:=temp;
           end;
         end;
     end;
    end;

    PROCEDURE get_files(var amfile:integer);
    var i,k : integer;
        dirinfo :searchrec;
    begin
         i:=0;
         k:=i;
         findfirst(mask, $3f, dirinfo);
         while doserror=0 do begin
               inc(i);
               files[i]:=dirinfo.name;
               findnext(dirinfo);
         end;
         amfile:=i+1;
       sort_it(k,amfile-1);
    end;

{}
PROCEDURE GO_ADD;
VAR  NUMREAD,NUMWRITTEN : word;
     x,y                : word;
     entries            : word;
     tid                : string[4];
     ypos               : byte;
     msize,
     size               : longint;
     path               : string;
     DirInfo            : searchrec;
     search1,search2    : string;
     k,
     amfile             : integer;
     i,
     errors             : byte;
     skip               : boolean;      {if a key allready exists}

     FUNCTION Nonly(name:string):string;  {only the name of a file}
     var x : integer;
     begin
      x:=pos('.',name);
      if x=0 then name:=copy(name,1,length(name))
         else name:=copy(name,1,x-1);
      x:=length(Name);
      while (name[x]<>'\') and (x>0) do dec(x);
      nonly:=copy(name,x+1,length(name));
     end;

     FUNCTION Eonly(name:string):string;  {only the extension of a file}
     var x : integer;
     begin
      x:=pos('.',name);
      if x=0 then Eonly:=''
         else Eonly:=copy(name,x+1,length(name)-x);
     end;


BEGIN
  ypos:=wherey-1;
  { See if file exists }
  if not seefile(filename) then begin
     entries:=0;
     assign(f1,filename); rewrite(f1,1);
     blockwrite(f1,id[1],3);
     close(f1);
  end;

  assign(f1,filename); reset(f1,1);

  up_case(mask);
  get_files(amfile);
  path:=mask;
  x:=length(path);
  while (path[x]<>'\') and (x>0) do dec(x);
  path:=copy(path,1,x);
  k:=1;
  repeat
   dirinfo.name:=files[k];
{   up_case(dirinfo.name);}

     search1:=Nonly(mask); { get name }
     search2:=Eonly(mask); { get extension }
     if (wildcomp(Nonly(dirinfo.name),Nonly(mask))) and { check with filemask }
        (wildcomp(Eonly(dirinfo.name),Eonly(mask))) then begin { ok! }

      decode(false,path+files[k],sprite);
      if gfx_error<>'' then begin
         writeln('Not a correct PCX file...');
      end else begin
         writeln('Processing file:'+files[k]+'             ');
         if (dirinfo.attr<>$10) and
            (files[k]<>filename) and
            (files[k]<>'tmplib.$00') then begin
                 assign(f1 , filename); reset(f1,1); {Open the library file}
                 assign(TEMP, 'tmplib.$00'); rewrite(TEMP,1);    {Open the temporary file}
                 blockread(f1 ,tid[1],3);
                 tid[0]:=#3;
                 if tid<>id then begin
                    writeln('Not a XAS file');
                    halt(1);
                 end;
{                 blockread(f1 ,entries, sizeof(entries)); {Read the amount of entries...}
                 inc(entries);
                 blockwrite(temp,id[1],3);
{                 blockwrite(temp,entries, sizeof(entries));     {Write the new amount of entries}
                 {Copy old data}
                    repeat
                       blockread(f1, bcopy1,  sizeof(bcopy1),numread);
                       blockwrite(temp,    bcopy1,  numread       ,numwritten)
                    until (numread=0) or (numwritten <> numread);
                 x:=0;
                 i:=0;
                 blockwrite(temp,mem[seg(sprite.adr^):ofs(sprite.adr^)+2],sprite.dtx*sprite.dty);

                 close(temp);          {close temporary file}
                 close(f1);      {close library-file}

                 {COPY TEMP TO LIB}
                 erase(f1);
                 rename(temp,filename);
         end;
         freemem(sprite.adr,(sprite.dtx*sprite.dty)+2);
    end;
   end;
   inc(k);
  until k>amfile;
  exit;
END;

{}
BEGIN
 textcolor(7); textbackground(0); clrscr;
 write('Charset Creator '+version+'   ');
 writeln('Totally FreeWare!  program by P.Bestebroer');
 write('--------------------------------------------------------------------------------');

 if (see_for('?')) or (paramcount=0) then begin
    writeln('Need some help?.... well try this:');
    writeln;
    writeln('CHMAKER <option1> <option2> ');
    writeln;
    writeln('  OPTIONS:');
    writeln('       +O<filename>       - name of the destination file');
    writeln('       +M<fileMASK>       - name(s) of file(s) to include to the');
    writeln('                             destination file.');
    writeln;
    halt(0);
 end;

 filename:='';
 mask:='*.pcx';
 filename:=pget_string('+o',1);
 mask:=pget_string('+m',1);

 if filename='' then begin
    writeln('No filename selected');
    halt(1);
 end;
 if mask='' then begin
    writeln('No filemask selected (ie.: "char*.pcx" ');
    halt(1);
 end;

 go_add;

END.

